diff --git a/Makefile b/Makefile
index 384bf79..b021cba 100644
--- a/Makefile
+++ b/Makefile
@@ -1,11 +1,27 @@
-# SeaBIOS build system
-#
-# Copyright (C) 2008-2010  Kevin O'Connor <kevin@koconnor.net>
-#
-# This file may be distributed under the terms of the GNU LGPLv3 license.
-
-# Program version
-VERSION=pre-0.6.2-$(shell date +"%Y%m%d_%H%M%S")-$(shell hostname)
+##
+## SeaBIOS build system with kconfig
+##
+## Copyright (C) 2011       Marc Bertens <mbertens@xs4all.nl>
+## Copyright (C) 2008-2010  Kevin O'Connor <kevin@koconnor.net>
+##
+## This file may be distributed under the terms of the GNU LGPLv3 license.
+## 
+export top			:= $(CURDIR)
+export srck			:= $(top)/tools/kconfig
+export obj			?= out
+export objutil			?= $(obj)/tools
+export objk			:= $(objutil)/kconfig
+
+export KERNELVERSION     	:= 0.6.2
+export KCONFIG_AUTOHEADER	:= $(obj)/aconfig.h
+export KCONFIG_AUTOCONFIG	:= $(obj)/auto.conf
+CONFIG_SHELL 			:= sh
+KBUILD_DEFCONFIG		:= src/defconfig
+UNAME_RELEASE			:= $(shell uname -r)
+DOTCONFIG 			?= .config
+KCONFIG_CONFIG			= $(DOTCONFIG)
+export KCONFIG_CONFIG
+HAVE_DOTCONFIG 			:= $(wildcard $(DOTCONFIG))
 
 # Output directory
 OUT=out/
@@ -47,7 +63,7 @@ CFLAGS16INC = $(CFLAGSSEG) -DMODE16=1 \
               $(call cc-option,$(CC),--param large-stack-frame=4,-fno-inline)
 CFLAGS16 = $(CFLAGS16INC) -g
 
-all: $(OUT) $(OUT)bios.bin
+all: $(obj)/aconfig.h $(OUT) $(OUT)bios.bin
 
 # Run with "make V=1" to see the actual compile commands
 ifdef V
@@ -55,16 +71,49 @@ Q=
 else
 Q=@
 endif
-
+strip_quotes = $(subst ",,$(subst \",,$(1)))
 OBJCOPY=objcopy
 OBJDUMP=objdump
 STRIP=strip
 
+## Too load the toolchain, test if the .xcompile is in the current dir, if so load it. 
+$(if $(wildcard .xcompile),$($(eval include .xcompile)),)
+
+CPP				:= $(CC) -x assembler-with-cpp -DASSEMBLY -E
+HOSTCC 				= gcc
+HOSTCXX 			= g++
+HOSTCFLAGS 			:= -I$(srck) -I$(objk) -g
+HOSTCXXFLAGS			:= -I$(srck) -I$(objk)
+
+$(obj)/aconfig.h:
+##	$(MAKE) menuconfig
+	$(MAKE) defconfig
+
+include $(HAVE_DOTCONFIG)
+
 .PHONY : all FORCE
 
 vpath %.c src vgasrc
 vpath %.S src vgasrc
 
+$(shell mkdir -p $(obj) $(objutil)/kconfig/lxdialog)
+
+# Program version
+ifdef CONFIG_LOCALVERSION
+ ifneq ($(CONFIG_LOCALVERSION),"")
+SEABIOS_EXTRA_VERSION	:= $(call strip_quotes,$(CONFIG_LOCALVERSION))
+ else
+SEABIOS_EXTRA_VERSION	= pre
+ endif
+VERSION			= $(SEABIOS_EXTRA_VERSION)-$(KERNELVERSION)-$(shell date +"%Y%m%d_%H%M%S")-$(shell hostname)
+else
+VERSION			= pre-$(KERNELVERSION)-$(shell date +"%Y%m%d_%H%M%S")-$(shell hostname)
+endif
+
+# This include must come _before_ the pattern rules below!
+# Order _does_ matter for pattern rules.
+include $(srck)/Makefile
+
 ################ Build rules
 
 # Verify the gcc configuration and test if -fwhole-program works.
@@ -195,7 +244,10 @@ $(OUT)ccode32flat.o: src/acpi-dsdt.hex
 
 ####### Generic rules
 clean:
-	$(Q)rm -rf $(OUT)
+	$(Q)rm -rf $(OUT)/version.* $(OUT)/asm-offsets.* $(OUT)/*.o $(OUT)/*.s $(OUT)/*.raw $(OUT)/*.d $(OUT)/*.elf $(OUT)/*.bin $(OUT)/*.lds $(OUT)/*.objdump 
+
+clean-dist:
+	$(Q)rm -rf $(OUT) .config
 
 $(OUT):
 	$(Q)mkdir $@

